module.exports = (options={}) ->
	log    = require './../helpers/log'
	path   = require 'path'
	pkg    = require './../package.json'
	isType = require './../helpers/isType'

	# defaults
	# ========
	dir =
		dist:    'dist'
		src:     'src'
		client:  'client'
		images:  'img'
		libs:    'libs' # things like bower_components
		server:  'server'
		scripts: 'scripts'
		styles:  'styles'
		views:   'views'

	file =
		spa:       'spa.html'
		appServer: 'routes' # app server dist entry file

	# options
	# =======
	formatOptions = ->
		['dist', 'src'].forEach (v1) ->
			options[v1] = {} if not isType.object options[v1]
			options[v1].dir = null if not isType.string options[v1].dir
			['client', 'server'].forEach (v2) ->
				# dir
				options[v1][v2] = {} if not isType.object options[v1][v2]
				options[v1][v2].dir = null if not isType.string options[v1][v2].dir
				return if v2 is 'server'
				# types dir
				['images', 'libs', 'scripts', 'styles', 'views'].forEach (v3) ->
					options[v1][v2][v3] = {} if not isType.object options[v1][v2][v3]
					options[v1][v2][v3].dir = null if not isType.string options[v1][v2][v3].dir
				# spa dist file
				if v1 is 'dist'
					options[v1][v2].spa = {} if not isType.object options[v1][v2].spa
					options[v1][v2].spa.file = null if not isType.string options[v1][v2].spa.file

	formatServerOptions	= -> # app server dist entry file
		options.dist.server.file = null if not isType.string options.dist.server.file

	formatPortOptions = -> # server ports
		options.ports = {} if not isType.object options.ports
		options.ports.server = null if not isType.number options.ports.server
		options.ports.reload = null if not isType.number options.ports.reload

	formatOptions()
	formatServerOptions()
	formatPortOptions()

	# init config
	# ===========
	config = {}

	# module
	# ======
	config.module =
		name: pkg.name
		task: 'rapid-build'
		dir: path.dirname "#{__dirname}.."

	# module config
	# =============
	config.module.config = {}
	config.module.config.dir  = "#{config.module.dir}/config"
	config.module.config.path = "#{config.module.config.dir}/config.json"

	# module tasks
	# ============
	config.module.tasks = {}
	config.module.tasks.dir = "#{config.module.dir}/tasks"

	# app
	# ===
	config.app =
		dir: process.cwd()

	# isDev
	# =====
	config.isDev = config.app.dir.indexOf("/#{config.module.name}") isnt -1

	# prefixes
	# ========
	config.prefix =
		distDir: 'rapid-build'
		tasks: if config.isDev then '' else 'rb-' # prefix for all tasks

	# ports
	# =====
	config.ports = {}
	config.ports.server = options.ports.server or 3000
	config.ports.reload = options.ports.reload or 3001

	# dirs
	# ====
	getDirs = (loc, isApp) ->
		o = {}
		if isApp
			o.dir           = options[loc].dir
			o.clientDir     = options[loc].client.dir
			o.clientImages  = options[loc].client.images.dir
			o.clientLibs    = options[loc].client.libs.dir
			o.clientScripts = options[loc].client.scripts.dir
			o.clientStyles  = options[loc].client.styles.dir
			o.clientViews   = options[loc].client.views.dir
			o.serverDir     = options[loc].server.dir

		dir: o.dir or dir[loc]
		client:
			dir: o.clientDir or dir.client
			images:
				dir: o.clientImages or dir.images
			libs:
				dir: o.clientLibs or dir.libs
			scripts:
				dir: o.clientScripts or dir.scripts
			styles:
				dir: o.clientStyles or dir.styles
			views:
				dir: o.clientViews or dir.views
		server:
			dir: o.serverDir or dir.server # gets deleted, see removeServerDir
			scripts:
				dir: o.serverDir or dir.scripts

	# dist
	# ====
	config.dist = {}
	config.dist.dir = options.dist.dir or dir.dist # only one dist.dir
	config.dist.rb  = getDirs 'dist'
	config.dist.app = getDirs 'dist', true

	# src
	# ===
	config.src = {}
	config.src.rb  = getDirs 'src'
	config.src.app = getDirs 'src', true

	# format config
	# =============
	formatConfig = (loc, src) ->
		cwd    = ''
		isSrc  = loc is 'src'
		isDist = loc is 'dist'
		if isSrc
			cwd = "#{config.module.dir}/" if src is 'rb'
			cwd = "#{config.app.dir}/" if src is 'app'
		loc = config[loc][src]
		for own k1, v1 of loc
			if k1 is 'dir'
				loc.dir = "#{cwd}#{v1}"
				continue
			for own k2, v2 of v1
				if k2 is 'dir'
					if isDist and src is 'rb'
						v1.dir = "#{loc.dir}/#{v2}/#{config.prefix.distDir}"
					else
						v1.dir = "#{loc.dir}/#{v2}"
				else
					v1[k2].dir = "#{v1.dir}/#{v2.dir}"

	formatConfig 'dist', 'rb'
	formatConfig 'dist', 'app'
	formatConfig 'src',  'rb'
	formatConfig 'src',  'app'

	# add spa
	# =======
	getDistSpa = ->
		spaFile = options.dist.client.spa.file or file.spa
		"#{config.dist.app.client.dir}/#{spaFile}"

	getSrcSpa = ->
		"#{config.src.rb.client.dir}/#{file.spa}"

	config.dist.app.client.spa =
		file: options.dist.client.spa.file or file.spa
		path: getDistSpa()

	config.src.rb.client.spa =
		file: file.spa
		path: getSrcSpa()

	# relative paths
	# ==============
	addAppRelative = (loc, loc2, type) ->
		a = options[loc].dir or dir[loc]
		b = options[loc][loc2].dir or dir[loc2]
		c = options[loc][loc2][type].dir or dir[type]
		"#{a}/#{b}/#{c}"

	config.src.rb.client.libs.relative  = "#{dir.src}/#{dir.client}/#{dir.libs}"
	config.src.app.client.libs.relative = addAppRelative 'src', 'client', 'libs'

	# final touch-ups
	# ===============
	formatDist = ->
		['app', 'rb'].forEach (v) ->
			delete config.dist[v].dir

	removeServerDir = ->
		['dist', 'src'].forEach (v1) ->
			['app', 'rb'].forEach (v2) ->
				config[v1][v2].server.scripts.dir = config[v1][v2].server.dir
				delete config[v1][v2].server.dir

	addToServerAppDist = ->
		config.dist.app.server.scripts.file = # app server dist entry file
			options.dist.server.file or file.appServer

		config.dist.app.server.scripts.path = # absolute path
			"#{config.app.dir}/#{config.dist.app.server.scripts.dir}"

	formatDist()         # only one dist.dir
	removeServerDir()    # server dir is server.scripts.dir
	addToServerAppDist() # add file and path

	# logs
	# ====
	# log.json config.dist, 'config.dist ='
	# log.json config.src, 'config.src ='
	# log.json config, 'config ='

	# return
	# ======
	config







