module.exports = (gulp, config) ->
	path  = require 'path'
	watch = require 'gulp-watch'
	log   = require './../helpers/log'

	# Helpers
	# =======
	getAppOrRb = (file) ->
		return 'rb' if file.path.indexOf(config.src.rb.dir) isnt -1
		'app'

	getClientOrServer = (file) ->
		return 'client' if file.path
			.indexOf(config.src[file.rbAppOrRb].server.scripts.dir) is -1
		'server'

	getRelative = (file) ->
		 dir = path.dirname file.relative
		 return '' if dir is '.'
		 "/#{dir}"

	getFileName = (file) ->
		fileName = path.basename file.path
		fileName

	getDistDir = (file, o={}) ->
		dir = config.dist[file.rbAppOrRb][file.rbClientOrServer][o.srcType].dir
		dir += file.rbRelative
		dir

	getDistPath = (file, o={}) ->
		fileName = file.rbFileName
		if o.extDist
			extSrc   = path.extname fileName
			fileName = fileName.replace extSrc, ".#{o.extDist}"
		dPath = "#{file.rbDistDir}/#{fileName}"
		dPath

	addFileProps = (file, o={}) -> # must be in order
		file.rbAppOrRb        = getAppOrRb file
		file.rbClientOrServer = getClientOrServer file
		file.rbRelative       = getRelative file
		file.rbFileName       = getFileName file
		file.rbDistDir        = getDistDir file, o
		file.rbDistPath       = getDistPath file, o
		file

	runTask = (taskName, file) ->
		require("./#{taskName}") gulp, config, file

	buildSpa = (task) ->
		buildSpaTask = "#{config.prefix.tasks}build-spa"
		return gulp.start buildSpaTask if not task or not task.then
		task.then -> gulp.start buildSpaTask

	events = (file, taskName, o={}) -> # add, change, unlink
		log.watch taskName, file
		return if not file
		return if not file.event
		return if not file.path
		return if not taskName
		taskName = 'clean-dist' if file.event is 'unlink'
		file     = addFileProps file, o
		task     = runTask taskName, file
		return task if file.event is 'change'
		buildSpa task

	getGlob = (loc, type, lang) ->
		globs = []
		['rb', 'app'].forEach (v1) ->
			['client', 'server'].forEach (v2) ->
				return if v2 is 'server' and type isnt 'scripts'
				globs.push config.glob[loc][v1][v2][type][lang]
		globs

	files =
		src:
			coffee: getGlob 'src', 'scripts', 'coffee'
			less:   getGlob 'src', 'styles',  'less'
			html:   getGlob 'src', 'views',   'html'

	gulp.task "#{config.prefix.tasks}watch", ->
		# scripts
		watch files.src.coffee, read:false, (file) ->
			events file, 'coffee', srcType:'scripts', extDist:'js'

		# styles
		watch files.src.less, read:false, (file) ->
			events file, 'less', srcType:'styles', extDist:'css'

		# views
		watch files.src.html, read:false, (file) ->
			events file, 'copy-html', srcType:'views', extDist:'html'

		# spa
		watch config.src.rb.client.spa.path, read:false, (file) ->
			log.watch 'spa', file
			buildSpa()


